package cache

import (
	"context"
	"errors"
	"github.com/redis/go-redis/v9"
	"github.com/rs/zerolog/log"
	"time"
)

type RedisCache struct {
	ctx context.Context
	rdb *redis.Client
}

func (r *RedisCache) Set(key string, value []byte, ttl time.Duration) error {
	return r.rdb.Set(r.ctx, key, value, ttl).Err()
}

func (r *RedisCache) Get(key string) ([]byte, bool) {
	val, err := r.rdb.Get(r.ctx, key).Bytes()
	if err != nil {
		if errors.Is(err, redis.Nil) {
			log.Error().Err(err).Str("key", key).Msg("Couldn't request key from cache.")
		}
		return nil, false
	} else {
		return val, true
	}
}

func (r *RedisCache) Remove(key string) {
	err := r.rdb.Del(r.ctx, key).Err()
	if err == nil {
		log.Error().Err(err).Str("key", key).Msg("Couldn't delete key from cache.")
	}
}

func NewRedisCache(opts *redis.Options) ICache {
	return &RedisCache{
		context.Background(),
		redis.NewClient(opts),
	}
}
